package labss.lang.il.builders;

import static labss.lang.il.model.ILTypesEnum.*;
import labss.lang.il.factories.ILAbstractFactory;
import labss.lang.il.factories.ILPrototypeManager;
import labss.lang.il.model.ILType;

/**
 * @author Riccardo Solmi
 */
public class ILModelBuilder extends DescendantOrSelfEntityBuilder implements ILBuilder {
	protected final ILAbstractFactory ef = ILAbstractFactory.instance;
	protected final ILPrototypeManager pm = ILPrototypeManager.instance();

	public void build(ILType type, boolean value) {
		build(ef.createBoolean(value));
	}
	public void build(ILType type, int value) {
		build(ef.createInt(value));
	}
	public void build(ILType type, String value) {
		build(ef.createVariable(value));
	}
	public void build(ILType type) {
		build(pm.create(type));
	}

	public void startBuild(ILType type) {
		startBuild(pm.create(type));
	}

	public void Bool(boolean value) {
		build(tBool, value);
	}
	public void Int(int value) {
		build(tInt, value);
	}
	public void Variable(String name) {
		build(tVariable, name);
	}

	public void NullExpression() {
		build(tNullExpression);
	}
	public void Empty() {
		build(tEmpty);
	}


	public void Block_() {
		startBuild(tBlock);
	}


	public void Addition_() {
		startBuild(tAddition);
	}

	public void And_() {
		startBuild(tAnd);
	}

	public void Assignment_() {
		startBuild(tAssignment);
	}

	public void Division_() {
		startBuild(tDivision);
	}

	public void Equals_() {
		startBuild(tEquals);
	}

	public void GreaterThan_() {
		startBuild(tGreaterThan);
	}

	public void IfElse_() {
		startBuild(tIfElse);
	}

	public void If_() {
		startBuild(tIf);
	}

	public void LessThan_() {
		startBuild(tLessThan);
	}

	public void Multiplication_() {
		startBuild(tMultiplication);
	}

	public void Not_() {
		startBuild(tNot);
	}

	public void Or_() {
		startBuild(tOr);
	}

	public void Subtraction_() {
		startBuild(tSubtraction);
	}

	public void While_() {
		startBuild(tWhile);
	}


	public void _Addition() {
		endBuild();
	}

	public void _And() {
		endBuild();
	}

	public void _Assignment() {
		endBuild();
	}

	public void _Block() {
		endBuild();
	}

	public void _Division() {
		endBuild();
	}

	public void _Equals() {
		endBuild();
	}

	public void _GreaterThan() {
		endBuild();
	}

	public void _If() {
		endBuild();
	}

	public void _IfElse() {
		endBuild();
	}

	public void _LessThan() {
		endBuild();
	}

	public void _Multiplication() {
		endBuild();
	}

	public void _Not() {
		endBuild();
	}

	public void _Or() {
		endBuild();
	}

	public void _Subtraction() {
		endBuild();
	}

	public void _While() {
		endBuild();
	}
}
