package labss.lang.il.visitors;

import java.io.PrintStream;

import labss.lang.il.model.ILEntity;
import labss.lang.il.model.ILTypesEnum;
import labss.lang.il.model.Literal;
import labss.lang.il.model.impl.BinaryExpression;
import labss.lang.il.model.impl.Assignment;
import labss.lang.il.model.impl.IfElse;
import labss.lang.il.model.impl.If;
import labss.lang.il.model.impl.Not;
import labss.lang.il.model.impl.Variable;
import labss.lang.il.model.impl.While;

/**
 * @author Riccardo Solmi
 */
public class PrettyPrintTypeSwitchBased {
	protected final PrintStream out;

	protected PrettyPrintTypeSwitchBased(PrintStream out) {
		this.out = out;
	}

	public static void prettyPrint(ILEntity entity, PrintStream out) {
		new PrettyPrintTypeSwitchBased(out).switchOn(entity);
	}

	public void switchOn(ILEntity entity) {
		switch (entity.getOrdinal()) {
		case ILTypesEnum.tEmpty_ord:
			out.println(";");
			break;
		case ILTypesEnum.tNullExpression_ord:
			out.print("null");
			break;
		case ILTypesEnum.tWhile_ord:
			out.print("while (");
			switchOn(((While) entity).getCondition());
			out.println(") {");
			switchOn(((While) entity).getTrueBody());
			out.println("}");
			break;
		case ILTypesEnum.tIf_ord:
			out.print("if (");
			switchOn(((If) entity).getCondition());
			out.println(") {");
			switchOn(((If) entity).getTrueBody());
			out.println("}");
			break;
		case ILTypesEnum.tIfElse_ord:
			out.print("if (");
			switchOn(((IfElse) entity).getCondition());
			out.println(") {");
			switchOn(((IfElse) entity).getTrueBody());
			out.println("} else {");
			switchOn(((IfElse) entity).getFalseBody());
			out.println("}");
			break;
		case ILTypesEnum.tAssignment_ord:
			switchOn(((Assignment) entity).getVariable());
			out.print(" = ");
			switchOn(((Assignment) entity).getExp());
			out.println(";");
			break;
		case ILTypesEnum.tBlock_ord:
			for (int i = 0; i < entity.size(); i++)
				switchOn(entity.get(i));
			break;
		case ILTypesEnum.tBool_ord:
			out.print(((Literal) entity).booleanValue());
			break;
		case ILTypesEnum.tInt_ord:
			out.print(((Literal) entity).intValue());
			break;
		case ILTypesEnum.tVariable_ord:
			out.print(((Variable) entity).getName());
			break;
		case ILTypesEnum.tAddition_ord:
		case ILTypesEnum.tSubtraction_ord:
		case ILTypesEnum.tMultiplication_ord:
		case ILTypesEnum.tDivision_ord:
		case ILTypesEnum.tEquals_ord:
		case ILTypesEnum.tGreaterThan_ord:
		case ILTypesEnum.tLessThan_ord:
		case ILTypesEnum.tAnd_ord:
		case ILTypesEnum.tOr_ord:
			switchOn(((BinaryExpression) entity).getExp1());
			out.print(entity.getType().opStr);
			switchOn(((BinaryExpression) entity).getExp2());
			break;
		case ILTypesEnum.tNot_ord:
			out.print(" ! ");
			switchOn(((Not) entity).getExp());
			break;
		}
	}
}
